@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.43';
+  $Math::Prime::Util::GMP::VERSION = '0.44';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -91,7 +91,7 @@ our @EXPORT_OK = qw(
                      random_maurer_prime_with_cert
                      random_shawe_taylor_prime_with_cert
                      seed_csprng is_csprng_well_seeded
-                     urandomb urandomr
+                     irand irand64 drand urandomb urandomm urandomr random_bytes
                    );
                    # Should add:
                    # nth_prime
@@ -195,7 +195,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.43
+Version 0.44
 
 
 =head1 SYNOPSIS
@@ -477,7 +477,7 @@ GMP integer, so may be large.  Typically this is not necessary, but
 cryptographic applications may prefer the ability to use this, and it
 allows repeatable test results.
 
-There is no check for duplicate bases.  Input sizes below 65-bits make
+There is no check for duplicate bases.  Input sizes below 65 bits make
 little sense for this function since L<is_prob_prime> is deterministic
 at that size.  For numbers of 65+ bits, the chance of duplicate bases
 is quite small.  The exponentiation approximation for the birthday
@@ -622,7 +622,8 @@ or 1 (probably prime), using the BPSW primality test (extra-strong variant).
 
 This function does the extra-strong BPSW test and nothing more.  That is,
 it will skip all pretests and any extra work that the L</is_prob_prime>
-test may add.
+test may add.  This saves some time if the input has no small factors, such
+as testing results that have been sieved.
 
 
 =head2 is_aks_prime
@@ -637,7 +638,7 @@ The particular method used is theorem 4.1 from Bernstein (2003).  This is
 substantially faster than the original AKS publication, the later version
 with improvements by Lenstra (sometimes called the V6 paper), or the later
 improvements of Voloch and Bornemann.  It is, by a large order, faster than
-any other known implementation as of early 2016.
+any other known implementation as of early 2017.
 
 For theoretical analysis of the primality task, AKS is extremely important.
 In practice, it is essentially useless.  Estimated run time for a 150 digit
@@ -645,6 +646,11 @@ input is over 2 days, making the case that while the algorithmic complexity
 I<growth> is polynomial, the constants are extremely high.  It will take
 years for for numbers that ECPP or APR-CL can prove in seconds.
 
+With the C<verbose> option set to 1, the chosen C<r> and C<s> values are
+printed before the test starts.  With C<verbose> set to 2 or higher, each
+of the C<s> tests results in a C<.> output as the test runs, allowing progress
+to be monitored.
+
 Typically you should use L</is_provable_prime> and let it decide the method.
 
 =head2 is_mersenne_prime
@@ -663,13 +669,12 @@ Takes a positive number C<n> as input and returns one of: 0 (definitely
 composite), 2 (definitely prime), or -1 (test does not indicate anything).
 This implements the Lucas-Lehmer-Riesel test for
 fast deterministic primality testing on numbers of the form C<k * 2^n - 1>.
+If the input is not of this form or if C<k E<gt>= 2^n> then C<-1> will be
+returned as the test does not apply.
 If C<k = 1> then this is a Mersenne number and the Lucas-Lehmer test is used.
-If the number is not of this form, or if C<k E<lt>= 2^n>, then C<-1> will
-be returned as the test does not apply.
-Otherwise, the LLR
-test is performed.  While not as fast as the Lucas-Lehmer test for Mersenne
-numbers, it is almost as fast as a single strong pseudoprime test (i.e.
-Miller-Rabin test) while giving a certain answer.
+Otherwise, the LLR test is performed.  While not as fast as the
+Lucas-Lehmer test for Mersenne numbers, it is almost as fast as a single
+strong pseudoprime test (i.e.  Miller-Rabin test) while giving a certain answer.
 
 =head2 is_proth_prime
 
@@ -677,8 +682,8 @@ Takes a positive number C<n> as input and returns one of: 0 (definitely
 composite), 2 (definitely prime), or -1 (test does not indicate anything).
 This applies Proth's theorem for fast Las Vegas
 primality testing on numbers of the form C<k * 2^n + 1>.
-If the number is not of this form, or if C<k E<lt>= 2^n>, then C<-1> will
-be returned as the test does not apply.
+If the input is not of this form or if C<k E<gt>= 2^n> then C<-1> will be
+returned as the test does not apply.
 Otherwise, a search is performed to find a quadratic nonresidue modulo C<n>.
 If none can be found after a brief search, C<-1> is returned as no conclusion
 can be reached.  Otherwise, Proth's theorem is checked which conclusively
@@ -897,7 +902,7 @@ hence the result is a probable prime (using BPSW).
 
 Returns the distances to the previous and next primes of the input C<n>.
 This is slightly more efficient than calling both L</prev_prime> and
-L</next_prime>, and returning the native integer distances can be more
+L</next_prime>, and returning the distances as native integers is more
 efficient with large inputs.
 
 If an optional second argument C<d> is given, and the input C<n> is larger
@@ -950,23 +955,23 @@ in the range.
 The returned prime has passed the C<is_prob_prime> (extra strong BPSW) test.
 
 The random prime functions use the internal CSPRNG for randomness.  This
-is currently ISAAC-32.
+is currently ISAAC-32 but will likely change to ChaCha20 in a later release.
 
 This corresponds to Mathematica's C<RandomPrime[{min,max}]> function.
 This is a superset of Pari's C<randomprime(n)> function, where our interval
-API is more useful for cryptographic functions.
+API is more convenient for cryptographic functions.
 
 =head2 random_maurer_prime
 
   say "random 512-bit proven prime: ", random_maurer_prime(512);
 
 Returns an n-bit proven prime using Ueli Maurer's FastPrime algorithm (1995).
-This results in uniform random selection of a proven prime, though with a
-small subset of the range not returned.
+This results in uniform random selection of a proven prime, though not every
+n-bit prime can be generated with this algorithm.
 
 C<undef> is returned if C<n> is less than C<2>.
-Internally the extra strong BPSW test has additionally been run on each
-intermediate and the final result, as a safety check.
+As a safety check, internally the extra strong BPSW test is additionally
+run on each intermediate and the final result.
 
 =head2 random_shawe_taylor_prime
 
@@ -979,8 +984,8 @@ It is a bit faster than Maurer's method but has a smaller subset of returned
 primes.
 
 C<undef> is returned if C<n> is less than C<2>.
-Internally the extra strong BPSW test has additionally been run on each
-intermediate and the final result, as a safety check.
+As a safety check, internally the extra strong BPSW test is additionally
+run on each intermediate and the final result.
 
 =head2 random_maurer_prime_with_cert
 
@@ -1077,7 +1082,7 @@ and C<u*x + v*y = d>.  This uses the extended Euclidian algorithm to compute
 the values satisfying Bézout's Identity.
 
 This corresponds to Pari's C<gcdext> function, which was renamed from
-C<bezout> out Pari 2.6.  The results will hence match L<Math::Pari/bezout>.
+C<bezout> in Pari 2.6.  The results will hence match L<Math::Pari/bezout>.
 
 =head2 chinese
 
@@ -1128,7 +1133,7 @@ For negative arguments, this matches Mathematica.  Pari does not implement
 the C<n E<lt> 0, k E<lt>= n> extension and instead returns C<0> for this
 case.  GMP's API does not allow negative C<k> but otherwise matches.
 L<Math::BigInt> does not implement any extensions and the results for
-C<n E<lt> 0, k > 0> are undefined.
+C<n E<lt> 0, k E<gt> 0> are undefined.
 
 =head2 bernfrac
 
@@ -1144,6 +1149,7 @@ Returns the Bernoulli number C<B_n> for an integer argument C<n>, as a
 string floating point.  An optional second argument indicates the number
 of significant digits to be used, with the result rounded.  The default
 is 40 digits.
+This corresponds to Pari's C<bernreal> function and.
 
 =head2 harmfrac
 
@@ -1151,6 +1157,7 @@ Returns the Harmonic number C<H_n> for an integer argument C<n>, as a
 rational number.  Two values are returned, the numerator and denominator.
 numbers are the sum of reciprocals of the first C<n> natural numbers:
 C<1 + 1/2 + 1/3 + ... + 1/n>.
+This corresponds to Mathematica's C<HarmonicNumber> function.
 
 =head2 harmreal
 
@@ -1264,7 +1271,8 @@ numbers, perfect numbers, etc.
 
 Takes a positive integer as input and returns the value of Ramanujan's tau
 function.  The result is a signed integer.
-This corresponds to Mathematica's C<RamanujanTau> function.
+This corresponds to Mathematica's C<RamanujanTau> function
+and Pari's C<ramanujantau> function.
 
 
 =head2 valuation
@@ -1369,9 +1377,9 @@ Perl version can produce C<partitions(10_000)> while with
 L<Math::Prime::Util::GMP> it can do C<partitions(220_000)>.  In contrast,
 in about 10 seconds Pari can solve C<numbpart(22_000_000)>.
 
-If you want the enumerated partitions, see L<Integer::Partition>.  It is
-very fast and uses an extremely memory efficient iterator.  It is not,
-however, practical for producing the partition I<number> for values
+If you want the enumerated partitions, see L<Math::Prime::Util/forpart>
+or L<Integer::Partition>.  These are fast and memory efficient iterators,
+but not practical for producing the partition I<number> for values
 over 100 or so.
 
 
@@ -1604,8 +1612,8 @@ Factoring will stop when the input is a prime, one factor has been found, or
 the algorithm fails to find a factor with the given smoothness.
 
 This is Pollard's C<p-1> method using a default smoothness of 5M and a
-second stage of C<B2 = 10 * B1>.  It can quickly find a factor C<p> of the input
-C<n> if the number C<p-1> factors into small primes.  For example
+second stage of C<B2 = 10 * B1>.  It can quickly find a factor C<p> of the
+input C<n> if the number C<p-1> factors into small primes.  For example
 C<n = 22095311209999409685885162322219> has the factor C<p = 3916587618943361>,
 where C<p-1 = 2^7 * 5 * 47 * 59 * 3137 * 703499>, so this method will find
 a factor in the first stage if C<B1 E<gt>= 703499> or in the second stage if
@@ -1630,8 +1638,8 @@ succeeded) or the original number (no factor was found).  In either case,
 multiplying @factors yields the original input.  An optional first stage
 smoothness factor (B1) may be given as the second parameter.  This will be
 the smoothness limit B1 for the first stage.
-Factoring will stop when the input is a prime, one factor has been found, or
-the algorithm fails to find a factor with the given smoothness.
+Factoring will stop when the input is a prime, one factor has been found,
+or the algorithm fails to find a factor with the given smoothness.
 
 
 
@@ -1750,8 +1758,7 @@ including the random prime functions.  Ideally this is 16-256 bytes of
 good entropy.
 
 Currently the CSPRNG is ISAAC-32, and the maximum number of seed bytes
-used is 1024.  It is possible the CSPRNG used will change in the future
-(thoughts include Salsa20, SHA2-256 counter, or SHA3-256 counter).
+used is 1024.  The CSPRNG will likely change to ChaCha20 in a later release.
 
 
 =head2 is_csprng_well_seeded
@@ -1773,6 +1780,16 @@ The result will be uniformly distributed between C<0> and C<2^b-1> inclusive.
 
 This is similar to the GMP function C<mpz_urandomb>.
 
+=head2 urandomm
+
+  $n = urandomm(100);    # random integer in [0,99]
+  $n = urandomm(1024);   # random integer in [0,1023]
+
+Given a positive integer C<n>, returns a random unsigned integer less than C<n>.
+The results will be uniformly distributed between C<0> and C<n-1> inclusive.
+
+This is similar to the GMP function C<mpz_urandomm>.
+
 =head2 urandomr
 
   $n  = urandomr(100, 110);        # Random number [100,110]
@@ -1785,6 +1802,77 @@ If C<low E<gt> high> then function will return C<undef>.
 Note that the range is inclusive, so C<low>, C<high>, and each integer
 between them have an equal probability of appearing.
 
+=head2 irand
+
+  $n32 = irand;     # random 32-bit integer
+
+Returns a random 32-bit integer using the CSPRNG.
+
+Performance is similar to
+L<Math::Random::MTwist/rand> and L<Math::Random::Xorshift>.
+It is somewhat faster than casting system C<rand> to a 32-bit int.
+It is noticibly faster than
+L<Math::Random::ISAAC>,
+L<Math::Random::ISAAC::XS>,
+L<Math::Random::MT>,
+L<Math::Random::MT::Auto>,
+and L<Crypt::PRNG>.
+
+=head2 irand64
+
+  $n64 = irand64;   # random 64-bit integer
+
+Returns a random 64-bit integer using the CSPRNG (on 64-bit Perl).
+
+=head2 drand
+
+  $f = drand;       # random floating point value in [0,1)
+  $r = drand(25);   # random floating point value in [0,25)
+
+Returns a random NV (Perl's native floating point) using the CSPRNG.
+
+The number of bits returned is equal to the mantissa bits of the NV type
+used for the Perl build, with a max of 64.  By default Perl uses doubles
+and the returned values have 53 bits.  If Perl is built with long double
+support and the long doubles have a larger mantissa, then more bits are
+used.
+
+This gives I<substantially> better quality random numbers than the default
+Perl C<rand> function.  Among other things, on modern Perl's, C<rand> uses
+drand48, which gives 32 bits of decent random values and 16 more bits of
+known patterns (e.g. the 48th bit alternates, the 47th has a period
+of 4, etc.).  There are much better choices for standard random number
+generators, such as the Mersenne Twister from L<Math::Random::MTwist>.
+
+Performance is similar to
+L<Math::Random::MTwist/rand> and L<Math::Random::Xorshift>.
+It is 1.5 - 2x slower than core C<rand> (as are the other modules).
+
+=head2 random_bytes
+
+  $str = random_bytes(32);     # 32 random bytes
+
+Given an unsigned number C<n> of bytes, returns a binary string filled with
+random data from the CSPRNG.  Performance for getting 256 byte strings:
+
+    Module/Method                  Rate   Type
+    -------------             ---------   ----------------------
+    Data::Entropy::Algorithms    2027/s   CSPRNG - AES Counter
+    Crypt::Random                6649/s   CSPRNG - /dev/urandom
+    Bytes::Random                9217/s   drand48
+    Bytes::Random::Secure       23043/s   CSPRNG - ISAAC
+    Math::Random::ISAAC::XS     58377/s   CSPRNG - ISAAC
+    rand+pack                   82977/s   drand48
+    Crypt::PRNG                298567/s   CSPRNG - Fortuna
+    Bytes::Random::XS          383354/s   drand48
+    ntheory                    770364/s   CSPRNG - ChaCha20
+    Math::Random::MTwist      1890151/s   Mersenne Twister
+    Math::Prime::Util::GMP    2045715/s   CSPRNG - ISAAC
+
+Each of the CSPRNG modules should be high quality.  There are no known
+flaws in any of ISAAC, AES CTR, ChaCha20, or Fortuna.  The industry
+seems to be standardizing onChaCha20 (e.g. BSD, Linux, TLS).
+
 
 =head1 SEE ALSO
 
@@ -1800,7 +1888,7 @@ this module directly is recommended.
 A Perl module with support for the strong Miller-Rabin test, strong
 Lucas-Selfridge test, the BPSW probable prime test, next_prime / prev_prime,
 the AKS primality test, and prime_count.  It uses L<Math::GMPz> to do all
-the calculations, so is faster than pure Perl bignums, but a little slower
+the calculations, so is faster than pure Perl bignums, but a lot slower
 than XS+GMP.  The prime_count function is only usable for very small inputs,
 but the other functions are quite good for big numbers.  Make sure to use
 version 0.05 or newer.
@@ -1826,12 +1914,6 @@ you want to use this.
 Open source APR-CL primality proof implementation.
 Fast primality proving, though without certificates.
 
-=item L<GMP-ECPP|http://sourceforge.net/projects/gmp-ecpp/>.
-An open source ECPP primality proving program.  Slower than this
-module's ECPP for all inputs when the large polynomial set from
-github is used.  Extremely slow once past 300 or so digits.
-There are now better alternatives.
-
 =back
 
 
